@@ -24,46 +24,11 @@
     GERMANY
     karlb@gmx.net                                                      
 */                                                                            
-#include <SDL.h>
-
 #include <assert.h>
 #include <stdlib.h>
 #include "SFont.h"
 
-static Uint32 GetPixel(SDL_Surface *Surface, Sint32 X, Sint32 Y)
-{
-   Uint8  *bits;
-   Uint32 Bpp;
-
-   assert(X>=0);
-   assert(X<Surface->w);
-   
-   Bpp = Surface->format->BytesPerPixel;
-   bits = ((Uint8 *)Surface->pixels)+Y*Surface->pitch+X*Bpp;
-
-   // Get the pixel
-   switch(Bpp) {
-      case 1:
-         return *((Uint8 *)Surface->pixels + Y * Surface->pitch + X);
-         break;
-      case 2:
-         return *((Uint16 *)Surface->pixels + Y * Surface->pitch/2 + X);
-         break;
-      case 3: { // Format/endian independent 
-         Uint8 r, g, b;
-         r = *((bits)+Surface->format->Rshift/8);
-         g = *((bits)+Surface->format->Gshift/8);
-         b = *((bits)+Surface->format->Bshift/8);
-         return SDL_MapRGB(Surface->format, r, g, b);
-         }
-         break;
-      case 4:
-         return *((Uint32 *)Surface->pixels + Y * Surface->pitch/4 + X);
-         break;
-   }
-
-   return -1;
-}
+#include "console.h"
 
 SFont_Font* SFont_InitFont(SDL_Surface* Surface)
 {
@@ -82,9 +47,9 @@ SFont_Font* SFont_InitFont(SDL_Surface* Surface)
 
     pink = SDL_MapRGB(Surface->format, 255, 0, 255);
     while (x < Surface->w) {
-	if (GetPixel(Surface, x, 0) == pink) { 
+	if (getpixel(Surface, x, 0) == pink) { 
     	    Font->CharPos[i++]=x;
-    	    while((x < Surface->w) && (GetPixel(Surface, x, 0)== pink))
+    	    while((x < Surface->w) && (getpixel(Surface, x, 0)== pink))
 		x++;
 	    Font->CharPos[i++]=x;
 	}
@@ -92,7 +57,7 @@ SFont_Font* SFont_InitFont(SDL_Surface* Surface)
     }
     Font->MaxPos = x-1;
     
-    pixel = GetPixel(Surface, 0, Surface->h-1);
+    pixel = getpixel(Surface, 0, Surface->h-1);
     SDL_UnlockSurface(Surface);
     SDL_SetColorKey(Surface, SDL_SRCCOLORKEY, pixel);
 
@@ -105,8 +70,23 @@ void SFont_FreeFont(SFont_Font* FontInfo)
     free(FontInfo);
 }
 
+/* Luola extension: Do a raw pixelcopy on areas where alpha is nonzero */
+static void nonzero_pixelcopy(Uint32 * srcpix, Uint32 * pixels, int w, int h,
+        int srcpitch, int pitch,Uint32 alphamask)
+{
+    int x,y;
+    for (y = 0; y < h; y++) {
+        for (x = 0; x < w; x++,pixels++,srcpix++) {
+            if ( (*srcpix & alphamask) )
+                *pixels = *srcpix;
+        }
+        pixels += pitch - w;
+        srcpix += srcpitch - w;
+    }
+}
+
 void SFont_Write(SDL_Surface *Surface, const SFont_Font *Font,
-		 int x, int y, const char *text)
+		 int x, int y, const char *text,int rawcopy)
 {
     const char* c;
     int charoffset;
@@ -118,7 +98,7 @@ void SFont_Write(SDL_Surface *Surface, const SFont_Font *Font,
     // these values won't change in the loop
     srcrect.y = 1;
     dstrect.y = y;
-    srcrect.h = dstrect.h = Font->Surface->h - 1;
+    srcrect.h = Font->Surface->h - 1;
 
     for(c = text; *c != '\0' && x <= Surface->w ; c++) {
 	charoffset = ((int) (*c - 33)) * 2 + 1;
@@ -128,14 +108,27 @@ void SFont_Write(SDL_Surface *Surface, const SFont_Font *Font,
 	    continue;
 	}
 
-	srcrect.w = dstrect.w = 
+	srcrect.w =
 	    (Font->CharPos[charoffset+2] + Font->CharPos[charoffset+1])/2 -
 	    (Font->CharPos[charoffset] + Font->CharPos[charoffset-1])/2;
 	srcrect.x = (Font->CharPos[charoffset]+Font->CharPos[charoffset-1])/2;
 	dstrect.x = x - (float)(Font->CharPos[charoffset]
 			      - Font->CharPos[charoffset-1])/2;
 
-	SDL_BlitSurface(Font->Surface, &srcrect, Surface, &dstrect); 
+    if(rawcopy) {
+        /* Do a raw pixel copy on areas where alpha!=0 */
+        if(dstrect.x<0) {
+            srcrect.x-=dstrect.x;
+            dstrect.x=0;
+        }
+        nonzero_pixelcopy((Uint32*)Font->Surface->pixels+Font->Surface->w+
+                srcrect.x,
+                (Uint32*)Surface->pixels+dstrect.y*Surface->w+dstrect.x,
+                srcrect.w,srcrect.h,
+                Font->Surface->w,Surface->w,Font->Surface->format->Amask);
+    } else {
+        SDL_BlitSurface(Font->Surface, &srcrect, Surface, &dstrect); 
+    }
 
 	x += Font->CharPos[charoffset+1] - Font->CharPos[charoffset];
     }
@@ -173,6 +166,6 @@ void SFont_WriteCenter(SDL_Surface *Surface, const SFont_Font *Font,
 		       int y, const char *text)
 {
     SFont_Write(Surface, Font, Surface->w/2 - SFont_TextWidth(Font, text)/2,
-	    	y, text);
+	    	y, text,0);
 }
 
