@@ -1,6 +1,6 @@
 /*
- * Luola - 2D multiplayer cavern-flying game
- * Copyright (C) 2001-2005 Calle Laakkonen
+ * Luola - 2D multiplayer cave-flying game
+ * Copyright (C) 2001-2006 Calle Laakkonen
  *
  * File        : animation.h
  * Description : This module handles all the animation and redraw timings
@@ -21,19 +21,31 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#ifndef ANIMATION_HEADER
-#define ANIMATION_HEADER
+#ifndef ANIMATION_H
+#define ANIMATION_H
 
-/* Functions */
+/* Reinitialize animation for a new level */
 extern void reinit_animation (void);
-extern void recalc_geometry(void);
+
+/* Rearrange screen update rectangles */
 extern void rearrange_animation (void);
+
+/* Recalculate player viewport geometry */
+extern void recalc_geometry(void);
+
+/* Get the size of a player viewport */
+extern SDL_Rect get_viewport_size(void);
+
+/* Pause/unpause the game */
 extern int pause_game (void);
+
+/* Start fading out a player viewport */
 extern void kill_plr_screen (int plr);
 
+/* Animate a single frame */
 extern void animate_frame (void);
 
-/* Globals */
+/* When >0, counts down to 0. When hits 0, the level ends */
 extern int endgame;
 
 #endif
