@@ -1,6 +1,6 @@
 /*
- * Luola - 2D multiplayer cavern-flying game
- * Copyright (C) 2001-2005 Calle Laakkonen
+ * Luola - 2D multiplayer cave-flying game
+ * Copyright (C) 2001-2006 Calle Laakkonen
  *
  * File        : console.h
  * Description : 
@@ -21,8 +21,8 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#ifndef L_CONSOLE_H
-#define L_CONSOLE_H
+#ifndef CONSOLE_H
+#define CONSOLE_H
 
 #include "SDL.h"
 
@@ -30,6 +30,13 @@
 #include "../config.h"
 #endif
 
+/* Player input device */
+struct Controller {
+    SDLKey keys[6];
+    SDL_Joystick *device;
+    int number;             /* 0=keyboard, rest are joystick number+1 */
+};
+
 /* The screen */
 extern SDL_Surface *screen;
 
@@ -37,52 +44,90 @@ extern SDL_Surface *screen;
 extern Uint32 col_black;
 extern Uint32 col_gray, col_grenade;
 extern Uint32 col_snow;
-extern Uint32 col_clay, col_clay2;      /* The secod clay colour is underwater clay */
+extern Uint32 col_clay, col_clay_uw;
 extern Uint32 col_default;
 extern Uint32 col_yellow;
 extern Uint32 col_red;
 extern Uint32 col_green;
 extern Uint32 col_blue;
+extern Uint32 col_cyan;
 extern Uint32 col_white;
-extern Uint32 col_transculent;
+extern Uint32 col_translucent;
 extern Uint32 col_rope;
 extern Uint32 col_plrs[4];
 extern Uint32 col_pause_backg;
 
-/* Initialize SDL library */
+/* Initialize video and audio */
 extern void init_sdl (void);
 
 /* Initialize screen */
 extern void init_video (void);
 
-/* Wait for enter */
+/* Wait for enter or joypad button */
 extern void wait_for_enter(void);
 
-/* Toggle between fullscreen mode */
+/* Open player joypads */
+extern void open_joypads();
+
+/* Close player joypads */
+extern void close_joypads();
+
+/* Toggle between fullscreen and windowed mode */
 extern void toggle_fullscreen(void);
 
-/* Graphics */
+/* Map color to RGBA value in current pixel format. Generates a */
+/* SDL_gfx compatible value if SDL_gfx is used. */
 extern Uint32 map_rgba(Uint8 r,Uint8 g,Uint8 b,Uint8 a);
+
+/* Decompose color to r,g,b and a values. */
+/* SDL_gfx format is used if enabled */
+extern void unmap_rgba(Uint32 c, Uint8 *r,Uint8 *g,Uint8 *b,Uint8 *a);
+
+/* Draw a box outline */
 extern void draw_box (int x, int y, int w, int h, int width, Uint32 color);
+
+/* Draw a filled box. If SDL_gfx is available, supports transparency */
 extern void fill_box (SDL_Surface *surface,int x ,int y,int w,int h,Uint32 color);
+
+/* Copy raw pixels from one surface to another */
 extern void pixelcopy (Uint32 * srcpix, Uint32 * pixels, int w, int h,
                        int srcpitch, int pitch);
-extern SDL_Surface *flip_surface (SDL_Surface * original);
+
+/* Make a deep copy of a surface */
 extern SDL_Surface *copy_surface (SDL_Surface * original);
+
+/* Make a new surface that is like the one given. */
+/* If w or h is 0, the new surface will be the same size as likethis */
+extern SDL_Surface *make_surface(SDL_Surface * likethis,int w,int h);
+
+/* Return the intersection of two rectangles */
 extern SDL_Rect cliprect (int x1, int y1, int w1, int h1, int x2, int y2,
                           int w2, int h2);
-extern char clip_line (int *x1, int *y1, int *x2, int *y2, int left, int top,
+
+/* Return a line segment inside a rectangle */
+extern int clip_line (int *x1, int *y1, int *x2, int *y2, int left, int top,
                        int right, int bottom);
+
+/* Multiply surface color values */
 extern void recolor (SDL_Surface * surface, float red, float green,
                      float blue, float alpha);
 
+/* Get a single pixel from a surface */
 extern Uint32 getpixel (SDL_Surface * surface, int x, int y);
 
+/* Resize a surface. A new surface is returned */
 extern SDL_Surface *zoom_surface (SDL_Surface *original, float aspect,float zoom);
 
+/* Display an error screen */
+extern void error_screen(const char *title, const char *exitsmg,
+        const char *message[]);
+
 #ifndef HAVE_LIBSDL_GFX
+/* Put a single pixel on screen */
 extern inline void putpixel (SDL_Surface * surface, int x, int y,
                              Uint32 color);
+
+/* Draw a line on screen */
 extern void draw_line (SDL_Surface * screen, int x1, int y1, int x2, int y2,
                        Uint32 pixel);
 #else
@@ -93,6 +138,6 @@ extern void draw_line (SDL_Surface * screen, int x1, int y1, int x2, int y2,
 
 /* Translate joystick events to keyboard events */
 extern void joystick_button (SDL_JoyButtonEvent * btn);
-extern void joystick_motion (SDL_JoyAxisEvent * axis, char plrmode);
+extern void joystick_motion (SDL_JoyAxisEvent * axis, int plrmode);
 
 #endif
