@@ -1,6 +1,6 @@
 /*
- * Luola - 2D multiplayer cavern-flying game
- * Copyright (C) 2003-2005 Calle Laakkonen
+ * Luola - 2D multiplayer cave-flying game
+ * Copyright (C) 2003-2006 Calle Laakkonen
  *
  * File        : font.c
  * Description : Font library abstraction
@@ -24,10 +24,8 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "defines.h"
 #include "console.h"
 #include "fs.h"
-#include "stringutil.h"
 #include "font.h"
 #include "startup.h"
 #include "parser.h"
@@ -96,21 +94,20 @@ static inline void set_proper_font (FontInfo * fi, FontSize size)
 static int load_font_cfg (const char *filename) {
     struct dllist *fontcfg;
     struct ConfigBlock *block;
-    CfgPtrType types[4];
-    void *pointers[4];
-    char *keys[4];
+    struct Translate tr[] = {
+        {"bigfont", CFG_STRING, &TruetypeFonts.bigfont},
+        {"smallfont", CFG_STRING, &TruetypeFonts.smallfont},
+        {"smallsize", CFG_INT, &TruetypeFonts.smallsize},
+        {"bigsize", CFG_INT, &TruetypeFonts.bigsize},
+        {0,0,0}
+    };
 
     fontcfg = read_config_file(filename,1);
     if(!fontcfg) return 1;
 
     block=fontcfg->data;
-    
-    keys[0]="bigfont";      types[0]=CFG_STRING;    pointers[0]=&TruetypeFonts.bigfont;
-    keys[1]="smallfont";    types[1]=CFG_STRING;    pointers[1]=&TruetypeFonts.smallfont;
-    keys[2]="smallsize";    types[2]=CFG_INT;       pointers[2]=&TruetypeFonts.smallsize;
-    keys[3]="bigsize";      types[3]=CFG_INT;       pointers[3]=&TruetypeFonts.bigsize;
 
-    translate_config(block->values,4,keys,types,pointers,0);
+    translate_config(block->values,tr,0);
 
     dllist_free(fontcfg,free_config_file);
     
@@ -144,13 +141,12 @@ int init_font (void) {
            if there isn't one there, check the font directory. */
         const char *fcfg;
         const char *filename;
-        fcfg = getfullpath (HOME_DIRECTORY, FONT_CFG_FILE);
+        fcfg = getfullpath (HOME_DIRECTORY, "fonts.cfg");
         if (load_font_cfg (fcfg)) {
-            fcfg = getfullpath (FONT_DIRECTORY, FONT_CFG_FILE);
+            fcfg = getfullpath (FONT_DIRECTORY, "fonts.cfg");
             if (load_font_cfg (fcfg)) {
-                printf(
-                    "Error: Couldn't load configuration file \"%s\"\n",
-                    FONT_CFG_FILE);
+                fprintf(stderr,
+                    "Error: Couldn't load font configuration file\n");
                 return 1;
             }
         }
@@ -187,8 +183,10 @@ int init_font (void) {
 #endif
     /* Initialize SFont */
     if (use_sfont) {
-        sfont_big=SFont_InitFont(load_image(FONT_DIRECTORY,"font1.png", 0,1));
-        sfont_small=SFont_InitFont(load_image(FONT_DIRECTORY,"font2.png", 0,1));
+        sfont_big=SFont_InitFont(load_image(
+                    getfullpath(FONT_DIRECTORY,"font1.png"), 0,T_ALPHA));
+        sfont_small=SFont_InitFont(load_image(
+                    getfullpath(FONT_DIRECTORY,"font2.png"), 0,T_ALPHA));
         MENU_SPACING = sfont_big->Surface->h;
     }
     /* Initialize font colors */
@@ -231,15 +229,8 @@ void putstring_direct (SDL_Surface * surface, FontSize size, int x, int y,
         if (color.r == 255 && color.g == 255 && color.b == 255) {
             SFont_Write (surface, font.sfont, x, y, text,0);
         } else {
-            fontsurface =
-                SDL_CreateRGBSurface (font.sfont->Surface->flags,
-                                  SFont_TextWidth(font.sfont, text),
-                                  font.sfont->Surface->h,
-                                  font.sfont->Surface->format->BitsPerPixel,
-                                  font.sfont->Surface->format->Rmask,
-                                  font.sfont->Surface->format->Gmask,
-                                  font.sfont->Surface->format->Bmask,
-                                  font.sfont->Surface->format->Amask);
+            fontsurface = make_surface(font.sfont->Surface,
+                     SFont_TextWidth(font.sfont, text), font.sfont->Surface->h);
             SFont_Write (fontsurface, font.sfont, 0, 0, text,1);
             recolor (fontsurface, color.r / 255.0, color.g / 255.0,
                      color.b / 255.0, 1.0);
@@ -254,8 +245,8 @@ void putstring_direct (SDL_Surface * surface, FontSize size, int x, int y,
     }
 }
 
-/* A convenience wrapper for putstring_direct		*/
-/* Centers the string horizontaly			*/
+/* A convenience wrapper for putstring_direct */
+/* Centers the string horizontaly */
 void centered_string (SDL_Surface * surface, FontSize size, int y,
                       const char *text, SDL_Color color)
 {
@@ -281,15 +272,8 @@ SDL_Surface *renderstring (FontSize size, const char *text, SDL_Color color)
     FontInfo font;
     set_proper_font (&font, size);
     if (use_sfont) {            /* SFont version */
-        fontsurface =
-            SDL_CreateRGBSurface (font.sfont->Surface->flags,
-                                  SFont_TextWidth(font.sfont, text),
-                                  font.sfont->Surface->h,
-                                  font.sfont->Surface->format->BitsPerPixel,
-                                  font.sfont->Surface->format->Rmask,
-                                  font.sfont->Surface->format->Gmask,
-                                  font.sfont->Surface->format->Bmask,
-                                  font.sfont->Surface->format->Amask);
+        fontsurface = make_surface(font.sfont->Surface,
+                     SFont_TextWidth(font.sfont, text), font.sfont->Surface->h);
         SFont_Write(fontsurface, font.sfont, 0, 0, text,1);
         if (color.r != 255 || color.g != 255 || color.b != 255) {
             recolor (fontsurface, color.r / 255.0, color.g / 255.0,
@@ -301,7 +285,7 @@ SDL_Surface *renderstring (FontSize size, const char *text, SDL_Color color)
     }
 }
 
-/* Get the height of the font                           */
+/* Get the height of the font */
 int font_height (FontSize size)
 {
     FontInfo font;
