@@ -1,6 +1,6 @@
 /*
- * Luola - 2D multiplayer cavern-flying game
- * Copyright (C) 2001-2005 Calle Laakkonen
+ * Luola - 2D multiplayer cave-flying game
+ * Copyright (C) 2001-2006 Calle Laakkonen
  *
  * File        : game.c
  * Description : Game configuration and initialization
@@ -27,9 +27,7 @@
 #include <dirent.h>
 #include <SDL.h>
 
-#include "defines.h"
 #include "startup.h"
-#include "stringutil.h"
 #include "console.h"
 #include "fs.h"
 #include "game.h"
@@ -47,129 +45,113 @@ static SDL_Surface *gam_filler;
 GameInfo game_settings;
 PerLevelSettings level_settings;
 GameStatus game_status;
-Uint8 game_loop;
+int game_loop;
 
-int init_game (void)
+static void load_game_config (void);
+
+/* Set default configuration values and load player screen filler images */
+void init_game (LDAT *miscfile)
 {
     int p;
-    LDAT *datafile;
-    datafile =
-        ldat_open_file (getfullpath (GFX_DIRECTORY, "misc.ldat"));
-    if(!datafile) return 1;
 
-    gam_filler = load_image_ldat (datafile, 1, 0, "FILLER", luola_options.videomode);
-    ldat_free (datafile);
+    gam_filler = load_image_ldat (miscfile, 1, T_OPAQUE, "FILLER", luola_options.videomode);
     if (gam_filler)
         recolor (gam_filler, 0.4, 0.4, 0.4, 1.0);
 
     /* Set up the default keys */
-    game_settings.buttons[0][0] = SDLK_w;
-    game_settings.buttons[0][1] = SDLK_s;
-    game_settings.buttons[0][2] = SDLK_a;
-    game_settings.buttons[0][3] = SDLK_d;
-    game_settings.buttons[0][4] = SDLK_LSHIFT;
-    game_settings.buttons[0][5] = SDLK_LCTRL;
+    game_settings.controller[0].keys[0] = SDLK_w;
+    game_settings.controller[0].keys[1] = SDLK_s;
+    game_settings.controller[0].keys[2] = SDLK_a;
+    game_settings.controller[0].keys[3] = SDLK_d;
+    game_settings.controller[0].keys[4] = SDLK_LSHIFT;
+    game_settings.controller[0].keys[5] = SDLK_LCTRL;
 
-    game_settings.buttons[1][0] = SDLK_UP;
-    game_settings.buttons[1][1] = SDLK_DOWN;
-    game_settings.buttons[1][2] = SDLK_LEFT;
-    game_settings.buttons[1][3] = SDLK_RIGHT;
-    game_settings.buttons[1][4] = SDLK_RSHIFT;
-    game_settings.buttons[1][5] = SDLK_RCTRL;
+    game_settings.controller[1].keys[0] = SDLK_UP;
+    game_settings.controller[1].keys[1] = SDLK_DOWN;
+    game_settings.controller[1].keys[2] = SDLK_LEFT;
+    game_settings.controller[1].keys[3] = SDLK_RIGHT;
+    game_settings.controller[1].keys[4] = SDLK_RSHIFT;
+    game_settings.controller[1].keys[5] = SDLK_RCTRL;
 
-    game_settings.buttons[2][0] = SDLK_i;
-    game_settings.buttons[2][1] = SDLK_k;
-    game_settings.buttons[2][2] = SDLK_j;
-    game_settings.buttons[2][3] = SDLK_l;
-    game_settings.buttons[2][4] = SDLK_y;
-    game_settings.buttons[2][5] = SDLK_h;
+    game_settings.controller[2].keys[0] = SDLK_i;
+    game_settings.controller[2].keys[1] = SDLK_k;
+    game_settings.controller[2].keys[2] = SDLK_j;
+    game_settings.controller[2].keys[3] = SDLK_l;
+    game_settings.controller[2].keys[4] = SDLK_y;
+    game_settings.controller[2].keys[5] = SDLK_h;
 
-    game_settings.buttons[3][0] = SDLK_KP8;
-    game_settings.buttons[3][1] = SDLK_KP5;
-    game_settings.buttons[3][2] = SDLK_KP4;
-    game_settings.buttons[3][3] = SDLK_KP6;
-    game_settings.buttons[3][4] = SDLK_KP1;
-    game_settings.buttons[3][5] = SDLK_KP2;
+    game_settings.controller[3].keys[0] = SDLK_KP8;
+    game_settings.controller[3].keys[1] = SDLK_KP5;
+    game_settings.controller[3].keys[2] = SDLK_KP4;
+    game_settings.controller[3].keys[3] = SDLK_KP6;
+    game_settings.controller[3].keys[4] = SDLK_KP1;
+    game_settings.controller[3].keys[5] = SDLK_KP2;
 
     for (p = 0; p < 4; p++) {
-        game_settings.controller[p] = Keyboard;
+        game_settings.controller[p].number = 0;
+        game_settings.controller[p].device = NULL;
     }
-    game_settings.ship_collisions = 1;
     game_settings.ls.indstr_base = 0;
     game_settings.base_regen = 1;
+    game_settings.ship_collisions = 1;
     game_settings.coll_damage = 1;
     game_settings.enable_smoke = 1;
     game_settings.ls.jumpgates = 0;
     game_settings.jumplife = 1;
+    game_settings.onewayjp = 1;
+    game_settings.soldiers = 15;
+    game_settings.helicopters = 5;
     game_settings.ls.turrets = 0;
     game_settings.ls.critters = 1;
     game_settings.ls.cows = 5;
     game_settings.ls.fish = 5;
     game_settings.ls.birds = 5;
     game_settings.ls.bats = 5;
-    game_settings.ls.soldiers = 15;
-    game_settings.ls.helicopters = 5;
     game_settings.ls.snowfall = 0;
     game_settings.ls.stars = 1;
     game_settings.endmode = 0;
-    game_settings.levelcount = 0;
     game_settings.levels = NULL;
-    game_settings.first_level = NULL;
-    game_settings.last_level = NULL;
-    game_settings.gravity_bullets = 1;
-    game_settings.wind_bullets = 0;
     game_settings.large_bullets = 0;
     game_settings.weapon_switch = 0;
     game_settings.eject = 1;
     game_settings.explosions = 1;
-    game_settings.holesize = 2;
     game_settings.recall = 0;
     game_settings.criticals = 0;
     game_settings.bigscreens = 1;
 
     game_settings.sounds = 0;
     game_settings.music = 0;
-    game_settings.playlist = 0;
+    game_settings.playlist = PLS_ORDERED;
     game_settings.sound_vol = 128;
     game_settings.music_vol = 128;
 
     /* Set the temporary settings */
     game_settings.mbg_anim = luola_options.mbg_anim;
 
-    return 0;
+    /* Load the configuration file */
+    load_game_config();
 }
 
+/* Reset player and game states for a new game */
 void reset_game (void)
 {
     int p;
     for (p = 0; p < 4; p++) {
-        game_settings.players_in[p] = ' ';
         game_status.wins[p] = 0;
         game_status.lifetime[p] = 0;
-        player_teams[p] = p;
     }
+    reset_players();
     game_settings.rounds = 5;
     game_settings.playmode = Normal;
     game_status.lastwin = 0;
 }
 
-void prematch_game (void)
-{
-    /* Randomize level */
-    int r, n;
-    n = rand () % (game_settings.levelcount);
-    for (r = -1; r < n; r++)
-        if (game_settings.levels->next == NULL)
-            game_settings.levels = game_settings.first_level;
-        else
-            game_settings.levels = game_settings.levels->next;
-}
-
-void apply_per_level_settings (LevelSettings * settings)
+/* Override current settings with the ones set in the level */
+void apply_per_level_settings (struct LevelSettings * settings)
 {
     level_settings = game_settings.ls;
     if (settings->override) {
-        LSB_Override *o=settings->override;
+        struct LSB_Override *o=settings->override;
         if(o->indstr_base>=0) level_settings.indstr_base=o->indstr_base;
         if(o->critters>=0) level_settings.critters=o->critters;
         if(o->stars>=0) level_settings.stars=o->stars;
@@ -180,8 +162,6 @@ void apply_per_level_settings (LevelSettings * settings)
         if(o->fish>=0) level_settings.fish=o->fish;
         if(o->birds>=0) level_settings.birds=o->birds;
         if(o->bats>=0) level_settings.bats=o->bats;
-        if(o->soldiers>=0) level_settings.soldiers=o->soldiers;
-        if(o->helicopters>=0) level_settings.helicopters=o->helicopters;
     }
 }
 
@@ -224,7 +204,7 @@ void game_statistics (void)
     data_line = 0;
     for (p = 0; p < 4; p++)
         if (players[p].state)
-            teams[player_teams[p]] = 1;
+            teams[get_team(p)] = 1;
     for (t = 0; t < 4; t++) {
         data_format[data_line] = 0;
         sprintf (data[data_line][0], "Team %d", t + 1);
@@ -233,7 +213,7 @@ void game_statistics (void)
             mostwins = 0;
             avglife = 0;
             for (p = 0; p < 4; p++) {
-                if (players[p].state && player_teams[p] == t) {
+                if (players[p].state && get_team(p) == t) {
                     data_line++;
                     data_format[data_line] = 1;
                     sprintf (data[data_line][0], "Player %d", p + 1);
@@ -331,10 +311,7 @@ void game_statistics (void)
 
 /* Draw the luola logo on all screen quarters.
  * Active player screens will just draw over it. */
-void fill_unused_player_screens (void)
-{
-    SDL_FillRect (screen, NULL, 0);
-
+void fill_player_screens (void) {
     if (gam_filler) {
         SDL_Rect rect;
         int p;
@@ -348,6 +325,8 @@ void fill_unused_player_screens (void)
                 rect.x -= screen->w;
             }
         }
+    } else {
+        SDL_FillRect (screen, NULL, 0);
     }
     SDL_UpdateRect (screen, 0, 0, 0, 0);
 }
@@ -363,9 +342,6 @@ void game_eventloop (void) {
             switch (Event.type) {
             case SDL_KEYDOWN:
                 /* Key down event */
-#ifdef CHEAT_POSSIBLE
-                if (cheat) cheatcode (Event.key.keysym.sym);
-#endif
                 if (Event.key.keysym.sym == SDLK_ESCAPE) return;
                 else if (Event.key.keysym.sym == SDLK_F1)
                     radars_visible = !radars_visible;
@@ -410,6 +386,8 @@ void game_eventloop (void) {
             case SDL_JOYAXISMOTION:
                 player_joyaxishandler (&Event.jaxis);
                 break;
+            case SDL_QUIT:
+                exit(0);
             default:
                 break;
             }
@@ -433,7 +411,7 @@ void save_game_config (void)
     int p, k;
     FILE *fp;
     const char *filename;
-    filename = getfullpath (HOME_DIRECTORY, CONF_FILE);
+    filename = getfullpath (HOME_DIRECTORY, "luola.cfg");
     fp = fopen (filename, "w");
     if (!fp) {
         printf ("Error! Cannot open file \"%s\" for writing\n", filename);
@@ -442,34 +420,33 @@ void save_game_config (void)
     /* Write controllers */
     fprintf (fp, "[controllers]\n");
     for (p = 0; p < 4; p++) {
-        fprintf (fp, "%d=%d\n", p, game_settings.controller[p]);
+        fprintf (fp, "%d=%d\n", p, game_settings.controller[p].number);
     }
     /* Write keys */
     for (p = 0; p < 4; p++) {
         fprintf (fp, "[keys%d]\n", p + 1);
         for (k = 0; k < 6; k++) {
-            fprintf (fp, "%d=%d\n", k, game_settings.buttons[p][k]);
+            fprintf (fp, "%d=%d\n", k, game_settings.controller[p].keys[k]);
         }
     }
     /* Write settings */
     fprintf (fp, "[settings]\n");
-    fprintf (fp, "ship_collisions=%d\n", game_settings.ship_collisions);
     fprintf (fp, "indestructable_base=%d\n", game_settings.ls.indstr_base);
     fprintf (fp, "regenerate_base=%d\n", game_settings.base_regen);
+    fprintf (fp, "ship_collisions=%d\n", game_settings.ship_collisions);
     fprintf (fp, "collision_damage=%d\n", game_settings.coll_damage);
     fprintf (fp, "jumpgates=%d\n", game_settings.ls.jumpgates);
     fprintf (fp, "jumplife=%d\n", game_settings.jumplife);
+    fprintf (fp, "onewayjp=%d\n", game_settings.onewayjp);
+    fprintf (fp, "soldiers=%d\n", game_settings.soldiers);
+    fprintf (fp, "helicopters=%d\n", game_settings.helicopters);
     fprintf (fp, "turrets=%d\n", game_settings.ls.turrets);
     fprintf (fp, "critters=%d\n", game_settings.ls.critters);
     fprintf (fp, "cows=%d\n", game_settings.ls.cows);
     fprintf (fp, "birds=%d\n", game_settings.ls.birds);
     fprintf (fp, "fish=%d\n", game_settings.ls.fish);
-    fprintf (fp, "soldiers=%d\n", game_settings.ls.soldiers);
-    fprintf (fp, "helicopters=%d\n", game_settings.ls.helicopters);
     fprintf (fp, "bats=%d\n", game_settings.ls.bats);
     fprintf (fp, "smoke=%d\n", game_settings.enable_smoke);
-    fprintf (fp, "bullet_gravity=%d\n", game_settings.gravity_bullets);
-    fprintf (fp, "bullet_wind=%d\n", game_settings.wind_bullets);
     fprintf (fp, "large_bullets=%d\n", game_settings.large_bullets);
     fprintf (fp, "bigscreens=%d\n", game_settings.bigscreens);
     fprintf (fp, "snowfall=%d\n", game_settings.ls.snowfall);
@@ -478,7 +455,6 @@ void save_game_config (void)
     fprintf (fp, "weapon_switch=%d\n", game_settings.weapon_switch);
     fprintf (fp, "eject=%d\n", game_settings.eject);
     fprintf (fp, "explosions=%d\n", game_settings.explosions);
-    fprintf (fp, "holesize=%d\n", game_settings.holesize);
     fprintf (fp, "recall=%d\n", game_settings.recall);
     fprintf (fp, "criticalhits=%d\n", game_settings.criticals);
 
@@ -493,54 +469,52 @@ void save_game_config (void)
 
 /* Parse configuration file settings block */
 static void parse_settings_block(struct dllist *values) {
-    CfgPtrType types[33];
-    void *pointers[33];
-    char *keys[33];
-
-    keys[0]="ship_collisions";      types[0]=CFG_INT; pointers[0]=&game_settings.ship_collisions;
-    keys[1]="indestructable_base";  types[1]=CFG_INT; pointers[1]=&game_settings.ls.indstr_base;
-    keys[2]="regenerate_base";      types[2]=CFG_INT; pointers[2]=&game_settings.base_regen;
-    keys[3]="collision_damage";     types[3]=CFG_INT; pointers[3]=&game_settings.coll_damage;
-    keys[4]="jumpgates";            types[4]=CFG_INT; pointers[4]=&game_settings.ls.jumpgates;
-    keys[5]="jumplife";             types[5]=CFG_INT; pointers[5]=&game_settings.jumplife;
-    keys[6]="turrets";              types[6]=CFG_INT; pointers[6]=&game_settings.ls.turrets;
-    keys[7]="critters";             types[7]=CFG_INT; pointers[7]=&game_settings.ls.critters;
-    keys[8]="cows";                 types[8]=CFG_INT; pointers[8]=&game_settings.ls.cows;
-    keys[9]="birds";                types[9]=CFG_INT; pointers[9]=&game_settings.ls.birds;
-    keys[10]="fish";                 types[10]=CFG_INT; pointers[10]=&game_settings.ls.fish;
-    keys[11]="soldiers";            types[11]=CFG_INT;pointers[11]=&game_settings.ls.soldiers;
-    keys[12]="helicopters";         types[12]=CFG_INT;pointers[12]=&game_settings.ls.helicopters;
-    keys[13]="bats";                types[13]=CFG_INT;pointers[13]=&game_settings.ls.bats;
-    keys[14]="smoke";               types[14]=CFG_INT;pointers[14]=&game_settings.enable_smoke;
-    keys[15]="bullet_gravity";      types[15]=CFG_INT;pointers[15]=&game_settings.gravity_bullets;
-    keys[16]="bullet_wind";         types[16]=CFG_INT;pointers[16]=&game_settings.wind_bullets;
-    keys[17]="snowfall";            types[17]=CFG_INT;pointers[17]=&game_settings.ls.snowfall;
-    keys[18]="stars";               types[18]=CFG_INT;pointers[18]=&game_settings.ls.stars;
-    keys[19]="endmode";             types[19]=CFG_INT;pointers[19]=&game_settings.endmode;
-    keys[20]="weapon_switch";       types[20]=CFG_INT;pointers[20]=&game_settings.weapon_switch;
-    keys[21]="eject";               types[21]=CFG_INT;pointers[21]=&game_settings.eject;
-    keys[22]="explosions";          types[22]=CFG_INT;pointers[22]=&game_settings.explosions;
-    keys[23]="holesize";            types[23]=CFG_INT;pointers[23]=&game_settings.holesize;
-    keys[24]="recall";              types[24]=CFG_INT;pointers[24]=&game_settings.recall;
-    keys[25]="criticalhits";        types[25]=CFG_INT;pointers[25]=&game_settings.criticals;
-    keys[26]="sounds";              types[26]=CFG_INT;pointers[26]=&game_settings.sounds;
-    keys[27]="music";               types[27]=CFG_INT;pointers[27]=&game_settings.music;
-    keys[28]="playlist";            types[28]=CFG_INT;pointers[28]=&game_settings.playlist;
-    keys[29]="music_volume";        types[29]=CFG_INT;pointers[29]=&game_settings.music_vol;
-    keys[30]="sound_volume";        types[30]=CFG_INT;pointers[30]=&game_settings.sound_vol;
-    keys[31]="large_bullets";       types[31]=CFG_INT;pointers[31]=&game_settings.large_bullets;
-    keys[32]="bigscreens";          types[32]=CFG_INT;pointers[32]=&game_settings.bigscreens;
-    translate_config(values,sizeof(types)/sizeof(CfgPtrType),keys,types,pointers,0);
+    struct Translate tr[] = {
+        {"indestructable_base", CFG_INT, &game_settings.ls.indstr_base},
+        {"regenerate_base", CFG_INT, &game_settings.base_regen},
+        {"ship_collisions", CFG_INT, &game_settings.ship_collisions},
+        {"collision_damage", CFG_INT, &game_settings.coll_damage},
+        {"jumpgates", CFG_INT, &game_settings.ls.jumpgates},
+        {"jumplife", CFG_INT, &game_settings.jumplife},
+        {"turrets", CFG_INT, &game_settings.ls.turrets},
+        {"critters", CFG_INT, &game_settings.ls.critters},
+        {"cows", CFG_INT, &game_settings.ls.cows},
+        {"birds", CFG_INT, &game_settings.ls.birds},
+        {"fish", CFG_INT, &game_settings.ls.fish},
+        {"soldiers", CFG_INT, &game_settings.soldiers},
+        {"helicopters", CFG_INT, &game_settings.helicopters},
+        {"bats", CFG_INT, &game_settings.ls.bats},
+        {"onewayjp", CFG_INT, &game_settings.onewayjp},
+        {"smoke", CFG_INT, &game_settings.enable_smoke},
+        {"snowfall", CFG_INT, &game_settings.ls.snowfall},
+        {"stars", CFG_INT, &game_settings.ls.stars},
+        {"endmode", CFG_INT, &game_settings.endmode},
+        {"weapon_switch", CFG_INT, &game_settings.weapon_switch},
+        {"eject", CFG_INT, &game_settings.eject},
+        {"explosions", CFG_INT, &game_settings.explosions},
+        {"recall", CFG_INT, &game_settings.recall},
+        {"criticalhits", CFG_INT, &game_settings.criticals},
+        {"sounds", CFG_INT, &game_settings.sounds},
+        {"music", CFG_INT, &game_settings.music},
+        {"playlist", CFG_INT, &game_settings.playlist},
+        {"music_volume", CFG_INT, &game_settings.music_vol},
+        {"sound_volume", CFG_INT, &game_settings.sound_vol},
+        {"large_bullets", CFG_INT, &game_settings.large_bullets},
+        {"bigscreens", CFG_INT, &game_settings.bigscreens},
+        {0,0,0}
+    };
+    translate_config(values,tr,0);
 }
 
 static void parse_controllers_block(struct dllist *values) {
     while(values) {
         struct KeyValue *pair=values->data;
         int plr=atoi(pair->key);
-        if(plr<0 || plr>3)
+        if(plr<0 || plr>3) {
             printf("No such player (%d)\n",plr);
-        else
-            game_settings.controller[plr] = atoi (pair->value);
+        } else {
+            game_settings.controller[plr].number = atoi (pair->value);
+        }
         values=values->next;
     }
 }
@@ -553,16 +527,16 @@ static void parse_keys_block(struct dllist *values,int player) {
         if(key<0 || key>5)
             printf("No such key (%d)\n",key);
         else
-            game_settings.buttons[player][key] = atoi (pair->value);
+            game_settings.controller[player].keys[key] = atoi (pair->value);
         values=values->next;
     }
 }
 
 /* Load game configuration */
-void load_game_config (void) {
+static void load_game_config (void) {
     struct dllist *gamecfg,*cfgptr;
     
-    gamecfg=read_config_file(getfullpath(HOME_DIRECTORY,CONF_FILE),0);
+    gamecfg=read_config_file(getfullpath(HOME_DIRECTORY,"luola.cfg"),0);
     if(!gamecfg) {
         printf("Couldn't load configuration file. Using built in defaults.\n");
         return;
@@ -582,3 +556,4 @@ void load_game_config (void) {
     }
     dllist_free(gamecfg,free_config_file);
 }
+
