@@ -1,6 +1,6 @@
 /*
- * Luola - 2D multiplayer cavern-flying game
- * Copyright (C) 2001-2005 Calle Laakkonen
+ * Luola - 2D multiplayer cave-flying game
+ * Copyright (C) 2001-2006 Calle Laakkonen
  *
  * File        : game.h
  * Description : Game configuration and initialization
@@ -21,20 +21,18 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#ifndef L_GAME_H
-#define L_GAME_H
+#ifndef GAME_H
+#define GAME_H
 
 #include "SDL.h"
 
+#include "console.h"
+#include "ldat.h"
 #include "lconf.h"
 
 #define PLAYMODE_COUNT 5
 
-typedef enum { Keyboard, Pad1, Pad2, Pad3, Pad4 } ControllerType;
-typedef enum { Forward, Backward, Left, Right, Weap1, Weap2 } Keys;
-typedef enum { Normal, OutsideShip, OutsideShip1, RndCritical,RndWeapon } Playmode;
-
-extern unsigned int Controllers;
+typedef enum { Normal, OutsideShip, OutsideShip1, RndCritical, RndWeapon } Playmode;
 
 struct dllist;
 
@@ -45,45 +43,40 @@ typedef struct {
     int critters;
     int cows, birds, fish, bats;
     int snowfall, stars;
-    int soldiers, helicopters;        /* These are the maximium number of hostile critters each player can have */
 } PerLevelSettings;
 
+typedef enum {JLIFE_SHORT,JLIFE_MEDIUM,JLIFE_LONG} Jumplife;
+
 typedef struct {
     int rounds;
-    char players_in[4];
     Playmode playmode;
     /* Controller options */
-    ControllerType controller[4];
-    SDLKey buttons[4][6];       /* Forward, Backward, Left,Right,Weap1,Weap2 */
-
+    struct Controller controller[4];
+    /* Levels */
     struct dllist *levels;
-    struct dllist *first_level;
-    struct dllist *last_level;
-    int levelcount;
     /* General game settings */
     int ship_collisions;
     int coll_damage;
-    int jumplife;
+    Jumplife jumplife;
+    int onewayjp;
     int enable_smoke;
     int endmode;
     int eject;
     int recall;
     int bigscreens;
     /* Weapon settings */
-    int gravity_bullets;
-    int wind_bullets;
     int large_bullets;
     int weapon_switch;
     int explosions;
-    int holesize;
     int criticals;
+    int soldiers, helicopters;
     /* Level settings */
-    PerLevelSettings ls;
+    PerLevelSettings ls; /* Use level_settings instead in game code */
     int base_regen;
     /* Audio settings */
     int sounds;
     int music;
-    int playlist;              /* Ordered, random */
+    enum {PLS_ORDERED,PLS_SHUFFLED} playlist;
     int sound_vol;             /* 0-128 */
     int music_vol;             /* 0-128 */
     /* Temporary settings */
@@ -97,14 +90,18 @@ typedef struct {
     int total_rounds;
 } GameStatus;
 
-/* Initialization */
-extern int init_game (void);
-extern void reset_game (void);
-extern void prematch_game (void);
-extern void apply_per_level_settings (LevelSettings * settings);
+/* Load datafiles */
+extern void init_game (LDAT *miscfile);
 
-/* Drawing */
-extern void fill_unused_player_screens (void);
+/* Reset game settings */
+extern void reset_game (void);
+
+/* Set level settings using game_settings.ls and the provided settings
+ * as sources */
+extern void apply_per_level_settings (struct LevelSettings * settings);
+
+/* Draw the filler image on all screen quadrants */
+extern void fill_player_screens (void);
 
 /* Game over statistics screen */
 extern void game_statistics (void);
@@ -112,14 +109,13 @@ extern void game_statistics (void);
 /* Ingame eventloop */
 extern void game_eventloop (void);
 
+/* Save game settings */
+extern void save_game_config (void);
+
 /* Some globals */
 extern GameInfo game_settings;
 extern PerLevelSettings level_settings;
 extern GameStatus game_status;
-extern Uint8 game_loop;
-
-/* Saving/loading configuration and level stuff */
-extern void save_game_config (void);
-extern void load_game_config (void);
+extern int game_loop;
 
 #endif
