@@ -1,6 +1,6 @@
 /*
- * Luola - 2D multiplayer cavern-flying game
- * Copyright (C) 2003-2005 Calle Laakkonen
+ * Luola - 2D multiplayer cave-flying game
+ * Copyright (C) 2003-2006 Calle Laakkonen
  *
  * File        : hotseat.h
  * Description : Hotseat game support code
@@ -25,7 +25,7 @@
 #define HOTSEAT_H
 
 /* Initialization */
-extern int init_hotseat (void);
+extern void init_hotseat (void);
 
 /* Start a normal hotseat game */
 extern void hotseat_game (void);
