@@ -1,6 +1,6 @@
 /*
- * Luola - 2D multiplayer cavern-flying game
- * Copyright (C) 2003-2005 Calle Laakkonen
+ * Luola - 2D multiplayer cave-flying game
+ * Copyright (C) 2003-2006 Calle Laakkonen
  *
  * File        : intro.h
  * Description : Intro and configuration screens
@@ -24,33 +24,21 @@
 #ifndef INTRO_H
 #define INTRO_H
 
+#include "ldat.h"
 #include "menu.h"
 
-/* Menu IDs */
-#define MENU_ID_INPUT			0x02
-#define MENU_ID_HOTSEAT			0x10
-
 /* Return values */
 #define INTRO_RVAL_STARTGAME	0x01
 #define INTRO_RVAL_EXIT		0x02
 
 /* Initialize intro menus and background animations */
-extern int init_intro (void);
+extern void init_intro (LDAT *miscfile);
 
 /* Intro menus. */
 extern int game_menu_screen (void);
 
 /* This is called called by menu callbacks. */
 /* It is defined here because it is used in other modules as well */
-extern int draw_input_icon (int x, int y, Uint8 align, int mid,
-                            MenuItem * item);
-/* This is called called by menu callbacks. */
-/* It is defined here because an other module needs it */
-extern int draw_team_icon (int x, int y, Uint8 align, int mid,
-                           MenuItem * item);
-
-/* Globals */
-SDL_Surface *keyb_icon[4], *pad_icon[4];
-
+extern int draw_input_icon (int x, int y, MenuAlign align, struct MenuItem * item);
 
 #endif
