@@ -1,6 +1,6 @@
 /*
- * Luola - 2D multiplayer cavern-flying game
- * Copyright (C) 2004-2005 Calle Laakkonen
+ * Luola - 2D multiplayer cave-flying game
+ * Copyright (C) 2004-2006 Calle Laakkonen
  *
  * File        : parser.h
  * Description : Generic configuration file parser
@@ -27,28 +27,36 @@
 #include "list.h"
 
 struct ConfigBlock {
-	char *title;
-	struct dllist *values;
+    char *title;
+    struct dllist *values;
 };
 
 struct KeyValue {
-	char *key;
-	char *value;
+    char *key;
+    char *value;
 };
 
-typedef enum {CFG_INT,CFG_FLOAT,CFG_DOUBLE,CFG_STRING} CfgPtrType;
+struct Translate {
+    char *key;
+    enum {CFG_INT,CFG_FLOAT,CFG_DOUBLE,CFG_STRING,CFG_MULTISTRING} type;
+    void *ptr;
+};
 
+/* Read a configuration file. Returns a list of ConfigBlocks */
 extern struct dllist *read_config_file(const char *filename,int quiet);
 
-extern void translate_config(struct dllist *values,int count,char **keys,CfgPtrType *types,void **pointers,int quiet);
+/* Read a configuration file from an SDL_RWops. len is the length of the */
+/* configuration file. If 0, RWops is read until EOF. */
+extern struct dllist *read_config_rw(SDL_RWops *rw,size_t len,int quiet);
 
+/* Extract data from a ConfigBlock */
+extern void translate_config(struct dllist *values,const struct Translate tr[],int quiet);
+
+/* Free a config block. Pass to dllist_free() */
 extern void free_config_file(void *data);
 
-/* Some useful functions */
-/* need to export ? */
-extern char *stripWhiteSpace(const char *str);
-extern int splitString(char *str,char delim,char **left,char **right);
-extern char *readLine(FILE *fp);
+/* Strip all whitespace characters from beginning and end */
+extern char *strip_white_space (const char *str);
 
 #endif
 
