@@ -1,6 +1,6 @@
 /*
- * Luola - 2D multiplayer cavern-flying game
- * Copyright (C) 2001-2005 Calle Laakkonen
+ * Luola - 2D multiplayer cave-flying game
+ * Copyright (C) 2001-2006 Calle Laakkonen
  *
  * File        : player.c
  * Description : Player information and animation
@@ -28,7 +28,6 @@
 
 #include "SDL.h"
 
-#include "defines.h"
 #include "fs.h"
 #include "console.h"
 #include "particle.h"
@@ -39,12 +38,12 @@
 #include "game.h"
 #include "special.h"
 #include "critter.h"
-#include "weather.h"
-#include "stringutil.h"
 #include "ship.h"
 
 #include "audio.h"
 
+#define SHIP_TURN_SPEED 0.15
+
 /* Internally used globals */
 static SDL_Surface *plr_weaponsel_bg;
 static SDL_Surface *plr_weapons[4];
@@ -52,78 +51,103 @@ static SDL_Surface *plr_weapons[4];
 static signed int player_message[4];
 static int plr_teamc[4];
 static Uint32 plr_healthbar_col, plr_healthbar_col2, plr_healthbar_col3,
-    plr_energybar_col, plr_blankbar_col;
+    plr_energybar_col,plr_noenergybar_col, plr_blankbar_col;
+static int player_teams[4];
 
 /* Exported globals */
 SDL_Surface *plr_messages[4];
-int player_teams[4];
 int plr_teams_left;
 int radars_visible;
 Player players[4];
-#ifdef CHEAT_POSSIBLE
-char cheat, cheat1;
-#endif
-
-/* Internally used functions */
-static void player_key_update(unsigned char plr);
 
 /* Initialize players */
-int init_players (void) {
+void init_players (LDAT *misc) {
     int p;
-    LDAT *misc;
     plr_healthbar_col = SDL_MapRGB (screen->format, 80, 98, 186);
     plr_healthbar_col2 = SDL_MapRGB (screen->format, 200, 200, 0);
     plr_healthbar_col3 = SDL_MapRGB (screen->format, 200, 0, 0);
     plr_energybar_col = SDL_MapRGB (screen->format, 186, 195, 195);
+    plr_noenergybar_col = SDL_MapRGB (screen->format, 186, 95, 95);
     plr_blankbar_col = SDL_MapRGB (screen->format, 106, 44, 123);
     for (p = 0; p < 4; p++) {
-        players[p].specialWeapon = WGrenade;
-        players[p].standardWeapon = SShot;
+        players[p].specialWeapon = 0;
+        players[p].standardWeapon = 0;
         /* Initialize message surface */
         plr_messages[p] = NULL;
         plr_weapons[p] = NULL;
     }
     radars_visible = 0;
     /* Load graphics */
-    misc = ldat_open_file (getfullpath (GFX_DIRECTORY, "misc.ldat"));
-    if(!misc) return 1;
-    plr_weaponsel_bg = load_image_ldat (misc, 1, 2, "WEAPONSEL", 0);
+    plr_weaponsel_bg = load_image_ldat (misc, 1, T_COLORKEY, "WEAPONSEL", 0);
     /* TODO: Load critical icons */
-    ldat_free (misc);
+}
 
-    return 0;
+/* Return the number of active players */
+int active_players(void) {
+    int p,a=0;
+    for(p=0;p<4;p++)
+        if(players[p].state != INACTIVE) a++;
+    return a;
+}
+
+/* Which team does the player belong to */
+int get_team(int plr) {
+    if(plr<0||plr>3) return -1;
+    return player_teams[plr];
+}
+
+/* Set which team a player belongs to */
+void set_team(int plr, int team) {
+    if(plr>=0 && plr<=3 && team>=0 && team<=3)
+        player_teams[plr] = team;
+}
+
+/* Check if two players are on the same team */
+int same_team(int plr1, int plr2) {
+    if((plr1<0||plr1>3) && (plr2<0||plr2>3)) return 1;
+    return player_teams[plr1] == player_teams[plr2];
+}
+
+/* Reset players for a new game */
+void reset_players(void) {
+    int p;
+    for(p=0;p<4;p++) {
+        players[p].state = INACTIVE;
+        player_teams[p] = p;
+    }
 }
 
 /* Prepare players for a new round */
 void reinit_players (void) {
     int p, unlucky = -1;
-    endgame = -1;
-    if (strcmp (game_settings.players_in, "    ") == 0) {
-        printf("Bug! reinit_players(): no players selected!\n");
-        abort();
+    if (active_players() == 0) {
+        fprintf(stderr, "Bug! reinit_players(): no players selected!\n");
+        exit(1);
     }
     if (game_settings.playmode == OutsideShip1) {
         do
             unlucky = rand () % 4;
-        while (game_settings.players_in[unlucky] == ' ');
+        while (players[unlucky].state == INACTIVE);
     }
+
+    clear_ships();
+    reinit_pilots();
+
     for (p = 0; p < 4; p++) {
         players[p].ship=NULL;
-        if (game_settings.players_in[p] == ' ') {
-            players[p].state=INACTIVE;
+        if (players[p].state == INACTIVE)
             continue;
-        }
+
         players[p].state = ALIVE;
         players[p].recall_cooloff = 0;
-        init_pilot(&players[p].pilot,p);
         memset (&players[p].controller, 0, sizeof (GameController));
         if (plr_weapons[p]) {
             SDL_FreeSurface (plr_weapons[p]);
             plr_weapons[p] = NULL;
         }
         if(game_settings.playmode == RndWeapon) {
-            players[p].standardWeapon = rand()%SWeaponCount;
-            players[p].specialWeapon = rand()%(WeaponCount-1)+1;
+            players[p].standardWeapon = rand()%normal_weapon_count()-1;
+            players[p].specialWeapon = rand()%(special_weapon_count()-1)+1;
         }
         players[p].weapon_select = 0;
         if (game_settings.playmode == OutsideShip1) {
@@ -146,14 +170,14 @@ void reinit_players (void) {
                 ship_critical (players[p].ship, 0);
         }
         if (p != unlucky) {
-            players[p].ship->x = lev_level.player_def_x[0][p];
-            players[p].ship->y = lev_level.player_def_y[0][p];
+            players[p].ship->physics.x = lev_level.player_def_x[0][p];
+            players[p].ship->physics.y = lev_level.player_def_y[0][p];
         }
         if (game_settings.playmode == OutsideShip
             || game_settings.playmode == OutsideShip1) {
             eject_pilot (p);
-            players[p].pilot.x = lev_level.player_def_x[1][p];
-            players[p].pilot.y = lev_level.player_def_y[1][p];
+            players[p].pilot.walker.physics.x = lev_level.player_def_x[1][p];
+            players[p].pilot.walker.physics.y = lev_level.player_def_y[1][p];
         }
         player_message[p] = 0;
         if (plr_messages[p]) {
@@ -164,7 +188,7 @@ void reinit_players (void) {
     /* Count the number of teams and the number of players in each of them */
     memset(plr_teamc,0,sizeof(int)*4);
     for (p = 0; p < 4; p++)
-        if (players[p].state)
+        if (players[p].state!=INACTIVE)
             plr_teamc[player_teams[p]]++;       /* Number of players per team */
     plr_teams_left = 0;
     for (p = 0; p < 4; p++)
@@ -184,8 +208,8 @@ void reinit_players (void) {
 static void draw_player_statusbar (int p) {
     SDL_Rect outline,health,energy;
     Uint32 healthcol;
-    outline.x = lev_rects[p].x + 19;
-    outline.y = lev_rects[p].y + cam_rects[p].h - 7;
+    outline.x = viewport_rects[p].x + 19;
+    outline.y = viewport_rects[p].y + cam_rects[p].h - 7;
     outline.w = cam_rects[p].w - 38;
     outline.h = 7;
 
@@ -210,12 +234,41 @@ static void draw_player_statusbar (int p) {
     health.w *= players[p].ship->health;
     energy.w *= players[p].ship->energy;
     SDL_FillRect (screen, &health, healthcol);
-    SDL_FillRect (screen, &energy, plr_energybar_col);
+    if(players[p].ship->special_cooloff || players[p].ship->energy <
+            special_weapon[players[p].ship->special].energy)
+        SDL_FillRect (screen, &energy, plr_noenergybar_col);
+    else
+        SDL_FillRect (screen, &energy, plr_energybar_col);
+}
+
+/* Draw a bar indicating rope length */
+static void draw_rope_bar(int plr) {
+    SDL_Rect bar;
+    bar.w = 3;
+    bar.h = cam_rects[plr].h/2;
+    bar.x = viewport_rects[plr].x + cam_rects[plr].w - bar.w - 10;
+    bar.y = viewport_rects[plr].y + cam_rects[plr].h/2 - bar.h/2;
+
+    fill_box(screen,bar.x,bar.y,bar.w,bar.h, col_rope);
+    draw_box(bar.x,bar.y,bar.w+1,bar.h+1,1, col_black);
+    bar.y += bar.h;
+    bar.w = 9;
+    bar.x -= 3;
+    bar.h = bar.h / (pilot_rope_maxlen - pilot_rope_minlen);
+    bar.y -= (players[plr].pilot.rope->nodelen-pilot_rope_minlen) * bar.h + bar.h/2;
+    fill_box(screen,bar.x,bar.y,bar.w,bar.h, col_rope);
+    draw_line(screen,bar.x,bar.y,bar.x+3,bar.y,col_black);
+    draw_line(screen,bar.x+6,bar.y,bar.x+9,bar.y,col_black);
+    draw_line(screen,bar.x,bar.y+bar.h,bar.x+2,bar.y+bar.h,col_black);
+    draw_line(screen,bar.x+6,bar.y+bar.h,bar.x+9,bar.y+bar.h,col_black);
+
+    draw_line(screen,bar.x,bar.y,bar.x,bar.y+bar.h,col_black);
+    draw_line(screen,bar.x+bar.w,bar.y,bar.x+bar.w,bar.y+bar.h,col_black);
 }
 
 /* Draw the weapon selection screen for player */
 static void draw_player_weaponselection (int plr) {
-    SDL_Rect rect = lev_rects[plr];
+    SDL_Rect rect = viewport_rects[plr];
     SDL_BlitSurface (plr_weaponsel_bg, NULL, screen, &rect);
     /* Update the string surface if necessary */
     if (players[plr].specialWeapon != players[plr].ship->special
@@ -223,8 +276,9 @@ static void draw_player_weaponselection (int plr) {
         players[plr].specialWeapon = players[plr].ship->special;
         SDL_FreeSurface (plr_weapons[plr]);
         plr_weapons[plr] =
-            renderstring (Smallfont, weap2str (players[plr].specialWeapon),
-                          font_color_green);
+            renderstring (Smallfont,
+                    special_weapon[players[plr].specialWeapon].name,
+                    font_color_green);
     }
     /* Blit text to screen */
     rect.x += cam_rects[plr].w/2 - plr_weapons[plr]->w / 2;
@@ -236,14 +290,14 @@ static void draw_player_weaponselection (int plr) {
 static void draw_player_message (int plr)
 {
     SDL_Rect rect;
-    rect.x = lev_rects[plr].x + cam_rects[plr].w/2 - plr_messages[plr]->w/2;
-    rect.y = lev_rects[plr].y + cam_rects[plr].h/2 - plr_messages[plr]->h/2;
+    rect.x = viewport_rects[plr].x + cam_rects[plr].w/2 - plr_messages[plr]->w/2;
+    rect.y = viewport_rects[plr].y + cam_rects[plr].h/2 - plr_messages[plr]->h/2;
     SDL_BlitSurface (plr_messages[plr], NULL, screen, &rect);
 }
 
 #if 0 /* TODO: Activate this when you have the critical icons */
 static  void draw_player_criticals(int plr) {
-  SDL_Rect rect=lev_rects[plr];
+  SDL_Rect rect=viewport_rects[plr];
   sDL_Rect src;
   int c;
   rect.y+=10;
@@ -266,10 +320,16 @@ void draw_player_hud(void) {
     for (p = 0; p < 4; p++) {
         if (players[p].state==ALIVE || players[p].state==DEAD) {
             if(players[p].ship) {
+                /* Draw ship status information */
                 draw_player_statusbar(p);
 #if 0
                 if(players[p].ship->criticals) draw_player_criticals(p);
 #endif
+            } else {
+                /* Draw pilot status information */
+                if(players[p].pilot.rope) {
+                    draw_rope_bar(p);
+                }
             }
             if (players[p].weapon_select)
                 draw_player_weaponselection (p);
@@ -287,13 +347,13 @@ void draw_radar (SDL_Rect rect, int plr) {
     if (players[plr].ship == NULL)
         return;
     for (p = 0; p < 4; p++)
-        if (p != plr && players[p].ship && players[p].ship->visible) {
-            d = atan2 (players[plr].ship->x - players[p].ship->x,
-                       players[plr].ship->y - players[p].ship->y);
-            dx = sin (d) * 7;
-            dy = cos (d) * 7;
-            dx2 = sin (d) * 17;
-            dy2 = cos (d) * 17;
+        if (p != plr && players[p].ship && players[p].ship->visible==1) {
+            d = atan2 (players[plr].ship->physics.y - players[p].ship->physics.y,
+                    players[plr].ship->physics.x - players[p].ship->physics.x);
+            dx = cos (d) * 7;
+            dy = sin(d) * 7;
+            dx2 = cos (d) * 17;
+            dy2 = sin(d) * 17;
             draw_line (screen, rect.x - dx, rect.y - dy, rect.x - dx2,
                        rect.y - dy2, col_plrs[p]);
         }
@@ -307,17 +367,11 @@ int find_player (struct Ship * ship) {
     return -1;
 }
 
-void player_cleanup (void) {
-    clean_ships ();
-}
-
 /*** PLAYER ANIMATION ***/
 void animate_players ()
 {
     struct dllist *ships;
     int n,newx,newy;
-    if (plr_teams_left == 0 && endgame == -1)
-        endgame = 30;           /* In case of tie game, end it */
     for (n = 0; n < 4; n++) {
         if (players[n].state!=ALIVE)
             continue;
@@ -329,30 +383,31 @@ void animate_players ()
             players[n].recall_cooloff--;
         /* Weapon selection */
         if (players[n].weapon_select) {
-            if (players[n].ship->onbase == 0) {
+            if (players[n].ship->physics.hitground != TER_BASE) {
                 players[n].weapon_select = 0;
             } else {
                 if (players[n].controller.axis[1] > 0
                     && players[n].ship->cooloff == 0) {
                     players[n].ship->special--;
-                    if (players[n].ship->special < 1)
-                        players[n].ship->special = WeaponCount - 1;
+                    if (players[n].ship->special < 0)
+                        players[n].ship->special = special_weapon_count() - 1;
                     players[n].ship->cooloff = 5;       /* We borrow the weapon cooloff */
                     players[n].ship->energy = 0;
                 } else if (players[n].controller.axis[1] < 0
                            && players[n].ship->cooloff == 0) {
                     players[n].ship->special++;
-                    if (players[n].ship->special == WeaponCount)
-                        players[n].ship->special = 1;
+                    if (players[n].ship->special == special_weapon_count())
+                        players[n].ship->special = 0;
                     players[n].ship->cooloff = 5;
                     players[n].ship->energy = 0;
                 }
             }
         }
         if (players[n].ship == NULL) {
-            /* End of game ? */
+            /* End game when pilot is on a base when there is only one */
+            /* team left and playmode is "last player must land" */
             if (endgame == -1 && game_settings.endmode
-                && players[n].pilot.onbase) {
+                && players[n].pilot.walker.physics.hitground==TER_BASE) {
                 if (plr_teams_left < 2)
                     endgame = 30;
             }
@@ -360,15 +415,16 @@ void animate_players ()
             ships = ship_list;
             while (ships) {
                 struct Ship *ship=ships->data;
-                if (players[n].pilot.x >= ship->x - 8
-                    && players[n].pilot.x <= ship->x + 8)
-                    if (players[n].pilot.y +
-                        players[n].pilot.sprite[0]->h >= ship->y - 8
-                        && players[n].pilot.y +
-                        players[n].pilot.sprite[0]->h - 2 <=
-                        ship->y + 8)
+                if (players[n].pilot.walker.physics.x >= ship->physics.x - 8
+                    && players[n].pilot.walker.physics.x <= ship->physics.x + 8)
+                    if (players[n].pilot.walker.physics.y >= ship->physics.y - 8
+                        && players[n].pilot.walker.physics.y <= ship->physics.y
+                        + 8)
                         if (find_player (ship) < 0
-                            && ship->eject_cooloff == 0) {
+                            && ship->eject_cooloff == 0)
+                        {
+                            /* Suitable ship found, remove pilot from level */
+                            remove_pilot(&players[n].pilot);
                             players[n].ship = ship;
                             players[n].recall_cooloff = 0;
                             if (players[n].ship->color == Grey)
@@ -377,31 +433,25 @@ void animate_players ()
                 ships = ships->next;
             }
         } else {
-            /* End of game ? */
+            /* End game when ship is on a base when there is only one */
+            /* team left and playmode is "last player must land" */
             if (endgame == -1 && game_settings.endmode
-                && players[n].ship->onbase) {
+                && players[n].ship->physics.hitground==TER_BASE) {
                 if (plr_teams_left < 2)
                     endgame = 30;
             }
         }
         /* Update camera position */
         if (players[n].ship) {
-            newx = Round(players[n].ship->x);
-            newy = Round(players[n].ship->y);
+            newx = Round(players[n].ship->physics.x);
+            newy = Round(players[n].ship->physics.y);
         } else {
-            newx = Round(players[n].pilot.x);
-            newy = Round(players[n].pilot.y);
-            if (players[n].pilot.parachuting) {
-                newy +=
-                    players[n].pilot.sprite[2]->h -
-                    players[n].pilot.sprite[0]->h;
-                newx +=
-                    players[n].pilot.sprite[2]->w / 2 -
-                    players[n].pilot.sprite[0]->w / 2;
-            }
+            newx = Round(players[n].pilot.walker.physics.x);
+            newy = Round(players[n].pilot.walker.physics.y);
         }
-        /* BUG: Sometimes with larger videomodes, setting y here will
-         * apparently clobber x */
+        /* If viewport width or height is greater than that of the */
+        /* level, x or y will be set to a negative value and luola */
+        /* will crash later. (Most likely in draw_stars) */
         cam_rects[n].x = newx - cam_rects[n].w/2;
         cam_rects[n].y = newy - cam_rects[n].h/2;
         if (cam_rects[n].x < 0)
@@ -415,15 +465,78 @@ void animate_players ()
     }
 }
 
+/* Generic input handling */
+void player_key_update (int plr) {
+    struct Ship *ship;
+    if(players[plr].state!=ALIVE) return;
+    if (players[plr].ship) {
+        if (players[plr].ship->state!=INTACT) {
+            if (players[plr].controller.weapon2)
+                eject_pilot (plr);
+            return;
+        }
+        if (players[plr].ship->frozen)
+            return;
+        /* Weapon selection */
+        if (players[plr].ship->physics.hitground==TER_BASE && players[plr].controller.axis[0] < 0) {
+            players[plr].weapon_select = 1;
+        } else
+            players[plr].weapon_select = 0;
+        /* Eject pilot */
+        if (players[plr].controller.axis[0] < 0
+            && players[plr].controller.weapon1 &&
+            game_settings.eject) {
+            eject_pilot (plr);
+            return;
+        }
+        if (players[plr].ship->darting)
+            return;
+        /* Remote control */
+        if (players[plr].ship->remote_control) {
+            if ((int) players[plr].ship->remote_control > 1)
+                ship = players[plr].ship->remote_control;
+            else
+                return; /* If you are being remote controlled, controls are disabled */
+        } else
+            ship = players[plr].ship;
+        if (players[plr].controller.weapon2 && ship != players[plr].ship) {
+            /* Disengage remote control */
+            remote_control(players[plr].ship, 0);
+            return;
+        }
+        /* Normal controls */
+        ship->thrust = players[plr].controller.axis[0] > 0;
+        ship->turn = players[plr].controller.axis[1] * SHIP_TURN_SPEED;
+        if ((ship->criticals & CRITICAL_LTHRUSTER) && ship->turn > 0)
+            ship->turn = 0;
+        else if ((ship->criticals & CRITICAL_RTHRUSTER) && ship->turn < 0)
+            ship->turn = 0;
+        ship->fire_weapon = players[plr].controller.weapon1
+            && !(ship->criticals & CRITICAL_STDWEAPON);
+        if (ship->fire_special_weapon != players[plr].controller.weapon2
+            && !(ship->criticals & CRITICAL_SPECIAL)) {
+            if(special_weapon[ship->special].singleshot) {
+                ship_fire_special(ship);
+                ship->fire_special_weapon = 0;
+                players[plr].controller.weapon2 = 0;
+            } else {
+                ship->fire_special_weapon = players[plr].controller.weapon2;
+            }
+        }
+    } else {
+        control_pilot (plr);
+    }
+}
+
 /* Keyboard handling */
 void player_keyhandler (SDL_KeyboardEvent * event, Uint8 type)
 {
     int p, b;
     SDLKey key = event->keysym.sym;
     for (p = 0; p < 4; p++) {   /* Check all 4 players */
-        if (players[p].state==ALIVE&&game_settings.controller[p]==Keyboard) {
-            for (b = 0; b < 6; b++) {       /* Each player has 6 buttons */
-                if (game_settings.buttons[p][b] == key) {
+        if (players[p].state==ALIVE&&game_settings.controller[p].number==0) {
+            for (b = 0; b < 6; b++) { /* Each player has 6 buttons */
+                if (game_settings.controller[p].keys[b] == key) {
                     switch (b) {
                     case 0:
                         players[p].controller.axis[0] = (type == SDL_KEYDOWN);
@@ -458,11 +571,11 @@ void player_joyaxishandler (SDL_JoyAxisEvent * axis)
     int p;
     signed char state;
     for (p = 0; p < 4; p++) {
-        if (players[p].state!=ALIVE||game_settings.controller[p]==Keyboard)
+        if (players[p].state!=ALIVE||game_settings.controller[p].number==0)
             continue;
-        if (game_settings.controller[p] - 1 != axis->which)
+        if (game_settings.controller[p].number - 1 != axis->which)
             continue;
-        state = abs (axis->value) > JSTRESHOLD;
+        state = abs (axis->value) > 16384;
         if (axis->axis == 0)    /* left/right */
             players[p].controller.axis[1] =
                 (axis->value > 0) ? -state : state;
@@ -479,9 +592,9 @@ void player_joybuttonhandler (SDL_JoyButtonEvent * button)
 {
     int p;
     for (p = 0; p < 4; p++) {
-        if (players[p].state!=ALIVE||game_settings.controller[p]==Keyboard)
+        if (players[p].state!=ALIVE||game_settings.controller[p].number==0)
             continue;
-        if (game_settings.controller[p] - 1 != button->which)
+        if (game_settings.controller[p].number - 1 != button->which)
             continue;
         switch (button->button) {
         case 1:
@@ -496,81 +609,6 @@ void player_joybuttonhandler (SDL_JoyButtonEvent * button)
     }
 }
 
-/* Generic input handling (this is where the real magic happens) */
-void player_key_update (unsigned char plr) {
-    struct Ship *ship;
-    if(players[plr].state!=ALIVE) return;
-    if (players[plr].ship) {
-        if (players[plr].ship->dead) {
-            if (players[plr].controller.weapon2)
-                eject_pilot (plr);
-            return;
-        }
-        if (players[plr].ship->frozen)
-            return;
-        /* Weapon selection */
-        if (players[plr].ship->onbase && players[plr].controller.axis[0] < 0) {
-            players[plr].weapon_select = 1;
-        } else
-            players[plr].weapon_select = 0;
-        if (players[plr].weapon_select) {
-        }
-        /* Eject pilot */
-        if (players[plr].controller.axis[0] < 0
-            && players[plr].controller.weapon1 &&
-            game_settings.eject) {
-            eject_pilot (plr);
-            return;
-        }
-        if (players[plr].ship->darting)
-            return;
-        /* Remote control */
-        if (players[plr].ship->remote_control) {
-            if ((int) players[plr].ship->remote_control > 1)
-                ship = players[plr].ship->remote_control;
-            else
-                return; /* If you are being remote controlled, controls are disabled */
-        } else
-            ship = players[plr].ship;
-        if (players[plr].controller.weapon2 && ship != players[plr].ship) {
-            /* Disengage remote control */
-            ship_fire (players[plr].ship, SpecialWeapon);
-            return;
-        }
-        /* Normal controls */
-        ship->thrust = players[plr].controller.axis[0] > 0;
-        ship->turn = players[plr].controller.axis[1] * TURN_SPEED;
-        if ((ship->criticals & CRITICAL_LTHRUSTER) && ship->turn < 0)
-            ship->turn = 0;
-        else if ((ship->criticals & CRITICAL_RTHRUSTER) && ship->turn > 0)
-            ship->turn = 0;
-        ship->fire_weapon = players[plr].controller.weapon1
-            && !(ship->criticals & CRITICAL_STDWEAPON);
-        if (ship->fire_special_weapon != players[plr].controller.weapon2
-            && !(ship->criticals & CRITICAL_SPECIAL)) {
-            switch (ship->special) {
-            /* Some weapons need to be fired only once */
-            case WCloak:
-            case WShield:
-            case WGhostShip:
-            case WAfterBurner:
-            case WRepair:
-            case WRemote:
-            case WAntigrav:
-                ship_fire (ship, SpecialWeapon);
-                ship->fire_special_weapon = 0;
-                players[plr].controller.weapon2 = 0;
-                break;
-            default:
-                ship->fire_special_weapon = players[plr].controller.weapon2;
-                break;
-            }
-        }
-    } else {
-        control_pilot (plr);
-    }
-}
-
 /* If a grenade explodes in the woods and there is nobody around to hear it,
  * does it really make a sound ? */
 int hearme (int x, int y) {
@@ -580,9 +618,9 @@ int hearme (int x, int y) {
     for (p = 0; p < 4; p++)
         if (players[p].state==ALIVE) {
             if (players[p].ship)
-                d = hypot (players[p].ship->x - x, players[p].ship->y - y);
+                d = hypot (players[p].ship->physics.x - x, players[p].ship->physics.y - y);
             else
-                d = hypot (players[p].pilot.x - x, players[p].pilot.y - y);
+                d = hypot (players[p].pilot.walker.physics.x - x, players[p].pilot.walker.physics.y - y);
             if (d < dist) {
                 nearest = p;
                 dist = d;
@@ -616,9 +654,10 @@ void set_player_message (int plr, FontSize size, SDL_Color color,
 /* This doesn't actually set player state to BURIED. That is set */
 /* in animation.c code after the player has been dead for FADE_STEP frames */
 /* so the fadeout animation can be drawn. */
-void buryplayer (int plr) {
+void kill_player (int plr) {
     if(players[plr].state!=ALIVE) {
-        printf("Bug! buryplayer(%d): Player is already dead or inactive!\n",plr);
+        fprintf(stderr,"Bug! %s(%d): Player is already dead or inactive!\n",
+                __func__,plr);
     } else {
         players[plr].state=DEAD;
         players[plr].ship=NULL;
@@ -630,36 +669,6 @@ void buryplayer (int plr) {
     }
 }
 
-/* Eject the pilot */
-void eject_pilot (int plr) {
-    pilot_any_ejected++;
-    if (players[plr].ship) {
-        players[plr].ship->fire_weapon = 0;
-        players[plr].ship->thrust = 0;
-        if (players[plr].ship->dead == 0)
-            players[plr].ship->turn = 0;
-        if (players[plr].ship->afterburn > 1) {
-            players[plr].ship->maxspeed = MAXSPEED;
-            players[plr].ship->afterburn = 1;
-        }
-        players[plr].pilot.x = players[plr].ship->x;   /* If you eject a player without a ship */
-        players[plr].pilot.y = players[plr].ship->y - players[plr].pilot.hy;  /* you'd better set the coordinates yourself... */
-        players[plr].ship->eject_cooloff = 70;
-        players[plr].ship = NULL;
-    }
-    memset (&players[plr].controller, 0, sizeof (GameController));
-    players[plr].weapon_select = 0;
-    players[plr].pilot.vector = makeVector (0, 3);
-    players[plr].pilot.rope = 0;
-    players[plr].pilot.walking = 0;
-    players[plr].pilot.updown = 0;
-    players[plr].pilot.lock = 0;
-    players[plr].pilot.parachuting = 0;
-    players[plr].pilot.maxspeed = MAXSPEED;
-    players[plr].pilot.hx = players[plr].pilot.sprite[0]->w / 2;
-    players[plr].pilot.hy = players[plr].pilot.sprite[0]->h;
-}
-
 /*** Recall the ship to the pilot ***/
 /* If the ship has been destroyed, create a new one */
 void recall_ship (int plr) {
@@ -682,18 +691,16 @@ void recall_ship (int plr) {
         }
         ships = ships->next;
     }
-    if (taken) {              /* If the ship is taken, it cannot be recalled */
+    if (taken) { /* If the ship is taken, it cannot be recalled */
         set_player_message (plr, Smallfont, font_color_green, 25,
                             "Ship taken, cannot recall");
         return;
     }
     /* Find good coordinates */
-    tx = players[plr].pilot.x;
-    ty = players[plr].pilot.y;
-    while (ty > players[plr].pilot.y - 20) {
-        if (ty <= 0
-            || (lev_level.solid[tx][ty] != TER_FREE
-                && lev_level.solid[tx][ty] != TER_TUNNEL)) {
+    tx = players[plr].pilot.walker.physics.x;
+    ty = players[plr].pilot.walker.physics.y-1;
+    while (ty > players[plr].pilot.walker.physics.y - 20) {
+        if (is_free(tx,ty)==0) {
             ty++;
             break;
         }
@@ -701,62 +708,25 @@ void recall_ship (int plr) {
     }
     /* Recall the ship */
     if (ship) {     /* Teleport the existing ship */
-        drop_jumppoint (ship->x, ship->y, plr + 20);
         drop_jumppoint (tx, ty, plr + 20);
-        ship->vector.x = 0;
-        ship->vector.y = 0;
+        drop_jumppoint (ship->physics.x, ship->physics.y, plr + 20);
+        ship->physics.vel.x = 0;
+        ship->physics.vel.y = 0;
     } else {        /* Ship probably destroyed, create a new one */
-        SpecialObj *jump;
+        struct SpecialObj *jump;
         ship =
             create_ship (Red + plr, players[plr].standardWeapon,
                          players[plr].specialWeapon);
-        ship->x = tx;
-        ship->y = ty;
+        ship->physics.x = tx;
+        ship->physics.y = ty;
         ship->energy = 0.3;
         ship->health = 0.5;
-        jump = malloc (sizeof (SpecialObj));
-        jump->x = tx - 16;
-        jump->y = ty - 16;
-        jump->owner = -1;
-        jump->frame = 0;
-        jump->timer = 0;
-        jump->age = JPSHORTLIFE;
-        jump->type = WarpExit;
-        jump->link = NULL;
-        addspecial (jump);
+        /* Create a decorative jumppoint */
+        jump = make_jumppoint(tx,ty,-1,1);
+        jump->life = jump->frames*1.5;
+        jump->hitship = NULL;
+        add_special(jump);
     }
-    players[plr].recall_cooloff=JPLONGLIFE*2;
+    players[plr].recall_cooloff=4*GAME_SPEED;
 }
 
-#ifdef CHEAT_POSSIBLE
-void cheatcode (SDLKey sym) {
-    int p;
-    switch (sym) {
-    case SDLK_HOME:            /* heal ships */
-        for (p = 0; p < 4; p++)
-            if (players[p].ship && players[p].ship->dead == 0) {
-                players[p].ship->health = 1;
-                set_player_message (p, Bigfont, font_color_red, 25,
-                                    "Health restored");
-            }
-        break;
-    case SDLK_PAGEUP:
-        for (p = 0; p < 4; p++)
-            if (players[p].ship && players[p].ship->dead == 0)
-                set_player_message (p, Bigfont, font_color_red, 25,
-                                    "Unlimited ammo");
-        cheat1 = 1;
-        break;
-    case SDLK_PAGEDOWN:
-        for (p = 0; p < 4; p++)
-            if (players[p].ship && players[p].ship->dead == 0)
-                set_player_message (p, Bigfont, font_color_red, 25,
-                                    "Limited ammo");
-        cheat1 = 0;
-        break;
-    default:
-        break;
-    }
-}
-#endif
-
